# -*- coding: utf8 -*-

from guichet.models import UserManager, GroupManager, Info_Sup, Info_Sup_Recv_File
from billetterie.utils.mkarchive import ZipArchive, TarArchive

import os
import json
from datetime import datetime
from decimal import Decimal


class CustomEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime):
            return obj.isoformat()
        if isinstance(obj, Decimal):
            return float(obj)
        return json.JSONEncoder.default(self, obj)


class UserDataDump(object):
    def __init__(self):
        # define if data was display in template or in json
        self.to_web = True

    def get_zip_archive(self, request, user):
        self.to_web = False
        with ZipArchive() as f:
            return self._get_archive(request, user, f)

    def get_tar_archive(self, request, user):
        self.to_web = False
        with TarArchive() as f:
            return self._get_archive(request, user, f)

    def get_user_data(self, user):
        res = {
            "email": user.email,
            "first_name": user.first_name,
            "last_name": user.last_name,
            "registrations": [self._get_registration_data(reg) for reg in user.registrations.all()],
            "permissions": self._get_user_perm(user),
        }
        if user.is_superuser:
            res["superuser"] = True
        if user.charte_date:
            res.update(
                {
                    "charte_date": user.charte_date,
                    "charte_version": user.charte_version,
                }
            )
        if not user.login_inp_net:
            res["compte_type"] = "exte"
        else:
            res.update(
                {
                    "compte_type": "INP-net",
                    "login_inp_net": user.login_inp_net,
                    "email_redirect": user.get_email(),
                    "ecole": user.ecole,
                    "promo": user.promo,
                    "adhesion_AE": user.is_adherent_ae,
                    "group_member": [str(grp) for grp in user.groupes.all()],
                    "group_manager": [str(grp) for grp in user.managed_clubs.all()],
                }
            )
        return res

    def _get_archive(self, request, user, archive):

        archive.addContent(CustomEncoder().encode(self.get_user_data(user)), "data.json")
        for info_sup in Info_Sup_Recv_File.objects.filter(registration__owner=user):
            if info_sup.value:
                file_path = "billet_{}/{}".format(info_sup.registration.id, os.path.basename(info_sup.value.path))
                archive.addFile(info_sup.value.path, file_path)
        return archive.get_response(request, "export_user_data")

    def _get_ipn_display(self, ipn):
        d = {
            "query_ipn": ipn.query,
            "valid_ipn": not ipn.flag,
            "status_ipn": ipn.payment_status,
            "payment_date": ipn.payment_date,
            "currency": ipn.mc_currency,
            "price": ipn.mc_gross,
            "buyer_email": ipn.payer_email,
        }
        if self.to_web:
            d["ipn"] = ipn
        return d

    def _get_option_data(self, opt):
        d = {
            "opt_name": opt.name,
            "opt_price": opt.price,
        }
        if opt.group:
            d["opt_group_name"] = opt.group_name
        return d

    def _get_info_data(self, info):
        d = {
            "info_name": info.reference.name,
            "info_state": info.state,
            "info_verbose_status": info.get_state_display(),
        }
        if info.reference.type_info != Info_Sup.INFO_SIMPLE and info.value:
            if info.reference.type_info != Info_Sup.INFO_FILE:
                d["info_value"] = info.value
            else:
                d["info_value"] = os.path.basename(info.value.path)
                d["info_url"] = info.value.url
        if self.to_web:
            d["info"] = info
        return d

    def _get_registration_data(self, reg):
        d = {
            "id": reg.id,
            "event_name": reg.ticket.event.name,
            "ticket_name": reg.ticket.name,
            "to": reg.get_real_payee(),
            "price": str(reg.get_real_amount()) + " €",
            "paid": reg.paid,
            "used": reg.used,
            "opposed": reg.opposed,
            "conditions_accepted": reg.conditions_accepted,
            "options": [self._get_option_data(opt) for opt in reg.options.all()],
            "info_sup": [self._get_info_data(info) for info in reg.info_sup_received.all()],
            "info_state": reg.get_info_state_display(),
            "ipn": [self._get_ipn_display(ipn) for ipn in reg.get_all_ipns()],
        }
        if reg.paid:
            d["payment_method"] = reg.get_payment_method_display()
            d["payment_date"] = reg.payment_date
        if reg.used:
            d["used_date"] = reg.used_date
        if reg.opposed:
            d["reason_opposed"] = reg.get_reason_opposed_display()
        if self.to_web:
            d["reg"] = reg
        return d

    def _data_user_perm(self, perm, grp=None):
        d = {
            "event_name": perm.event.name,
            "event_id": perm.event.pk,
            "access_description": perm.get_access_display(),
        }
        if self.to_web:
            d["event"] = perm.event
        if isinstance(perm, UserManager):
            d["access_type"] = "people"
        else:
            d["group_name"] = str(grp)
            d["access_type"] = "group_manager" if perm.bureau_only else "group"
        return d

    def _get_user_perm(self, user):
        res = []
        for perm in UserManager.objects.filter(user=user):
            res.append(self._data_user_perm(perm))

        for grp in user.groupes.all():
            for perm in GroupManager.objects.filter(group=grp, bureau_only=False):
                res.append(self._data_user_perm(perm, grp))

        for grp in user.managed_clubs.all():
            for perm in GroupManager.objects.filter(group=grp, bureau_only=True):
                res.append(self._data_user_perm(perm, grp))
        return res
